# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/lectures/14_augment.ipynb.

# %% auto 0
__all__ = ['act_gr', 'summary', 'CapturePreds', 'capture_preds', 'rand_erase', 'RandErase', 'rand_copy', 'RandCopy']

# %% ../nbs/lectures/14_augment.ipynb 2
import random
import fastcore.all as fc
from functools import partial

from datasets import load_dataset

import torch
from torch import distributions, nn, tensor, optim
from torch.nn import init
import torch.nn.functional as F
import torchvision.transforms.functional as TF
from torch.optim import lr_scheduler
from torcheval.metrics import MulticlassAccuracy

from .conv import def_device
from .datasets import inplace, show_images, show_image
from .learner import MetricsCB, DeviceCB, ProgressCB, DataLoaders, Learner, TrainLearner, SingleBatchCB, MomentumLearner, lr_find, Callback, to_cpu
from .activations import Hooks
from .init import conv, GeneralRelu, init_weights, ActivationStats, set_seed, BatchTransformCB
from .sgd import BatchSchedCB
from .resnet import ResBlock

set_seed(42)

# %% ../nbs/lectures/14_augment.ipynb 6
act_gr = partial(GeneralRelu, leak=0.1, sub=0.4)

# %% ../nbs/lectures/14_augment.ipynb 16
def _flops(x, h, w):
    if x.dim() < 3: return x.numel()
    if x.dim() == 4: return x.numel()*h*w
    raise Exception()

@fc.patch
def summary(self:Learner):
    res = '|Module|Input|Output|Num params|MFlops|\n|--|--|--|--|--|\n'
    totp, totf = 0, 0
    def _f(hook, mod, inp, outp):
        nonlocal res,totp,totf
        nparams = sum(o.numel() for o in mod.parameters())
        totp += nparams
        *_,h,w = outp.shape
        flops = sum(_flops(o, h, w) for o in mod.parameters())/1e6
        totf += flops
        res += f'{type(mod).__name__}|{tuple(inp[0].shape)}|{tuple(outp.shape)}|{nparams}|{flops:.1f}|\n'
    with Hooks(self.model, _f) as hooks: self.fit(1, lr=1, train=False, cbs=SingleBatchCB())
    print(f"Tot params: {totp}; MLFLOPS: {totf:.1f}")
    if fc.IN_NOTEBOOK:
        from IPython.display import Markdown
        return Markdown(res)
    else: print(res)

# %% ../nbs/lectures/14_augment.ipynb 42
class CapturePreds(Callback):
    def before_fit(self, learn): self.all_inps, self.all_preds, self.all_targs = [], [], []
    def after_batch(self, learn):
        self.all_inps.append(to_cpu(learn.batch[0]))
        self.all_preds.append(to_cpu(learn.preds))
        self.all_targs.append(to_cpu(learn.batch[1]))
    def after_fit(self, learn):
        self.all_preds, self.all_targs, self.all_inps = map(torch.cat, [self.all_preds, self.all_targs, self.all_inps])

# %% ../nbs/lectures/14_augment.ipynb 43
@fc.patch
def capture_preds(self:Learner, cbs=None, inps=False):
    cp = CapturePreds()
    self.fit(1, train=False, cbs=[cp] + fc.L(cbs))
    res = cp.all_preds,cp.all_targs
    if inps: res = res+(cp.all_inps,)
    return res

# %% ../nbs/lectures/14_augment.ipynb 59
def _rand_erase(x, pct, xm, xs, mn, mx):
    szx = int(pct * x.shape[-2])
    szy = int(pct * x.shape[-1])
    stx = int(random.random() * (1 - pct) * x.shape[-2])
    sty = int(random.random() * (1 - pct) * x.shape[-1])
    init.normal_(x[:,:,stx:stx+szx, sty:sty+szy])
    x.clamp_(mn, mx)

# %% ../nbs/lectures/14_augment.ipynb 62
def rand_erase(x, pct=0.2, max_num=4):
    xm, xs, mn, mx = x.mean(), x.std(), x.min(), x.max()
    num = random.randint(0, max_num)
    for i in range(num): _rand_erase(x, pct, xm, xs, mn, mx)
    return x

# %% ../nbs/lectures/14_augment.ipynb 64
class RandErase(nn.Module):
    def __init__(self, pct=0.2, max_num=4):
        super().__init__()
        self.pct, self.max_num = pct, max_num
    def forward(self, x): return rand_erase(x, self.pct, self.max_num)

# %% ../nbs/lectures/14_augment.ipynb 76
def _rand_copy1(x, pct):
    szx = int(pct * x.shape[-2])
    szy = int(pct * x.shape[-1])
    stx1 = int(random.random() * (1 - pct) * x.shape[-2])
    sty1 = int(random.random() * (1 - pct) * x.shape[-1])
    stx2 = int(random.random() * (1 - pct) * x.shape[-2])
    sty2 = int(random.random() * (1 - pct) * x.shape[-1])
    x[:, :, stx1:stx1+szx, sty1:sty1+szy] = x[:, :, stx2:stx2+szx, sty2:sty2+szy]

# %% ../nbs/lectures/14_augment.ipynb 78
def rand_copy(x, pct=0.2, max_num = 4):
    num = random.randint(0, max_num)
    for i in range(num): _rand_copy1(x, pct)
    return x

# %% ../nbs/lectures/14_augment.ipynb 80
class RandCopy(nn.Module):
    def __init__(self, pct=0.2, max_num=4):
        super().__init__()
        self.pct, self.max_num = pct, max_num
    def forward(self, x): return rand_copy(x, self.pct, self.max_num)
